__author__= 'Doble Engineering'

import glob
import subprocess
import logging
import logfiles
import os
import argparse
from envlib import Envlib, EnvironmentError

"""
MSOHFD-987

23Jun2016
These are the changes done to create the paritions as part of the upgrade of the firmware
I will try to document what all trivial things are done here probably this will/should going to the wiki


Typical installation process is a zip file is given to the DTA.
DTA transfers the zip file using the messages(21h and 22h)
tos run M7_fwupdate.sh <downloaded zip file>
M7_fwupdate.sh is from the firmware which is already running on the M7
example 1.2.6.6 M7_fwupdate.sh will be consuming the 1.2.7.12 zip file for the upgrade
M7_fwupdate.sh will be shipping the zip file to the other hosts
and run the M7_sysUpdater.py(this file) will be called to proceed with the installation

For creating the partitions, for which the packages are required to be installed before running
the actual updater which used to be M7_sysUpdater.py(this file)

This is a puzzle, where before running M7_sysUpdater.py, some libraries need to be installed but
the file fwupdate.sh will call is M7_sysUpdater.py
For this purpose, the "self extracting file" concept is used and installer.bin is created in which
the original zip is embedded.


as of this writing this file should be in 1.2.7.13 release
so in the 1.2.7.13 release a zip file would contain
a. installer.bin
b. 1.2.7.13 manifest file
c. 1.2.7.13 checksum file
d. M7_sysUpdater.py [which is this file]

this file will just call the installer.bin
installer.bin extracts the zip file from itself, installs the libraries, runs the python updater
which is used be this file(M7_sysUpdater.py) and renamed as actualupdater.py

"""

logbasename="preupdate.log"

INVALID_PARTITION_SET = -1
PARTITION_SET_ONE = 1
PARTITION_SET_TWO = 2


def RebootM7():
    subprocess.call(['wget','http://192.168.42.54/mess08.shtml'])


def DumpEnvironmentVariables(env,LOGFILE):
    LOGFILE.write(str(env) + '\n')

def EstablishPartitions(LOGFILE):
    if not os.path.isdir('/media/mmcblk0p1/root1'):
        os.mkdir('/media/mmcblk0p1/root1');

    if not os.path.isdir('/media/mmcblk0p1/root2'):
        os.mkdir('/media/mmcblk0p1/root2');

    if not os.path.isdir('/media/mmcblk0p2/root1'):
        os.mkdir('/media/mmcblk0p2/root1');

    if not os.path.isdir('/media/mmcblk0p2/root2'):
        os.mkdir('/media/mmcblk0p2/root2');


def SelectPartitions(env,LOGFILE):

    active_part_string = env.getenv('active_partitions')
    if not active_part_string:
        #Variable is not present, first time install
        active_part = INVALID_PARTITION_SET
        env.setenv('active_partitions',str(INVALID_PARTITION_SET))
    else:
        active_part = int(active_part_string)

    LOGFILE.write("active part is " + str(active_part) + "\n")

    if active_part == PARTITION_SET_ONE or active_part == INVALID_PARTITION_SET:
        expected_part = PARTITION_SET_TWO
    else:
        expected_part = PARTITION_SET_ONE

    env.setenv('expected_partitions',str(expected_part))
    if not env.saveenv():
        LOGILE.write('Failed to save the u-boot environment')

    LOGFILE.write("selecting part " + str(expected_part) + " as the target\n")

    return expected_part



def main():
    arg = argparse.ArgumentParser(description='Calls M7 Firmware Updater')
    arg.add_argument('-g', default=1, help='1 if calling in the foreground, 0 if calling in the background')
    arg.add_argument('-f')
    args = arg.parse_args()

#M7_fwupdate.sh extracted the zip file into tmp_fw_update directory
#just call the installer.bin file to start the installation
    
#create the logfile in non-volatile partition
    nvlogging  = logfiles.SetLogging(logbasename)
    nvlogging.CheckNVMount()
    logprefix = nvlogging.GetPrefix()
    logpath = logprefix + "/" + logbasename
    errorlog = "sysupdateerror.log"
#could be in JFFS2 or the mmcblk0p3 partition
    LOGFILE=open(logpath, "w+")

    
    try:
        env = Envlib()
    except EnvironmentError:
        LOGFILE.write("failed to read u-boot environment variables")
        LOGFILE.close()
        RebootM7()
        return -1; #Won't get here, but to be clear, we don't continue on

    DumpEnvironmentVariables(env,LOGFILE)

    LOGFILE.write("starting the updater which will call the installer \n")
    installer = glob.glob("tmp_fw_update/*.bin")

    EstablishPartitions(LOGFILE)

    part_set = SelectPartitions(env,LOGFILE)

    argfile = open(logprefix + '/updater_arguments.txt','w')
    argfile.write("-p " + str(part_set) + '\n')
    argfile.write("-g " + str(args.g) + '\n')
    argfile.close();

#there should be only one installer
#due to lack of space delete the original zip file
    cmd = "rm -rf *.zip;chmod +x %s;%s 2>%s/%s" % (installer[0], installer[0], logprefix, errorlog)
    LOGFILE.write("running cmd %s\n" % cmd)
    LOGFILE.flush()
    p1 = subprocess.Popen([cmd], shell=True)
    status = p1.wait()
    if status != 0:
        LOGFILE.write("%s fail \n" % cmd)
        LOGFILE.close()
        RebootM7()
        return -1
    
    LOGFILE.close()

# Standard boilerplate to call the main() function to begin
# the program.
if __name__ == '__main__':
    main()
    
